@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 import re
 import json
-import ctypes
 from .scw import SCW
 from .trie import Trie
 
@@ -34,6 +33,13 @@ FEATSET_ZH = (
     "c1", "c9", "c2", "c8", "c3", "c7"
 )
 
+TDEF = Trie.insert({}, [_DEF_LABEL, _DEF_LABEL], 1.)
+TDEF = Trie.insert(TDEF, [_BEOS_LABEL, _DEF_LABEL], 0.1)
+TDEF = Trie.insert(TDEF, [_DEF_LABEL, _BEOS_LABEL], 0.1)
+
+EDEF = Trie.insert({}, [_DEF_LABEL], 0.1)
+EDEF = Trie.insert(EDEF, [_BEOS_LABEL], 0.)
+
 
 class Token():
     __slots__ = ("c", "f", "l", "t")
@@ -45,7 +51,7 @@ class Token():
         self.t = t
 
     def __repr__(self):
-        return '<c=%s, f=%s, l=%s, t=%s>' % (self.c, self.f, self.l, self.t)
+        return "<c=%s, f=%s, l=%s, t=%s>" % (self.c, self.f, self.l, self.t)
 
 
 class RakutenMA(object):
@@ -59,8 +65,8 @@ class RakutenMA(object):
         """
         self.model = model
         self.scw = SCW(phi, c)
-        self.scw.mu = model.get('mu', {})
-        self.scw.sigma = model.get('sigma', {})
+        self.scw.mu = model.get("mu", {})
+        self.scw.sigma = model.get("sigma", {})
 
         self.ctype_func = self.ctype_ja_default_func
         self.build_ctype_map(CTYPE_JA_PATTERNS)
@@ -71,12 +77,12 @@ class RakutenMA(object):
     def set_tag_scheme(self, scheme):
         self.tag_scheme = scheme
 
-    def save(self, filename, float_repr='.6f'):
+    def save(self, filename, float_repr=".6f"):
         json.encoder.FLOAT_REPR = lambda o: format(o, float_repr)
-        json.dump(self.model, open(filename, 'w'))
+        json.dump(self.model, open(filename, "w"))
 
     def load(self, filename):
-        self.set_model(json.load(open(filename, 'r')))
+        self.set_model(json.load(open(filename, "r")))
 
     @staticmethod
     def string2hash(_str):
@@ -89,8 +95,14 @@ class RakutenMA(object):
         """
         _hash = 0
         for char in _str:
-            _hash = (ctypes.c_int(_hash << 5).value - _hash) + ord(char)
-            _hash = ctypes.c_int(_hash).value
+            shifted = _hash << 5
+            if shifted & 0x80000000:
+                shifted = -(0x100000000 - shifted)
+            _hash = (shifted - _hash) + ord(char)
+            if _hash >= 0x80000000:
+                _hash -= 0x100000000
+            elif _hash <= -0x80000000:
+                _hash = int("-0x" + hex(_hash)[-8:], 16)
         return _hash
 
     def create_hash_func(self, bits):
@@ -119,10 +131,7 @@ class RakutenMA(object):
             <list> csent
         """
         csent = [Token(l=_BEOS_LABEL)]  # BOS
-
-        for char in _input:
-            csent.append(Token(c=char, t=self.ctype_func(char)))
-
+        csent += [Token(c=char, t=self.ctype_func(char)) for char in _input]
         csent.append(Token(l=_BEOS_LABEL))  # EOS
         return csent
 
@@ -134,10 +143,11 @@ class RakutenMA(object):
         Return:
             <list> csent
         """
-        _t = lambda i: csent[i] if len(csent) > i >= 0 else Token()
+        csent_length = len(csent)
+        _t = lambda i: csent[i] if csent_length > i >= 0 else Token()
 
         # feature hashing function
-        _f = self.hash_func if hasattr(self, 'hash_func') else lambda x: x
+        _f = self.hash_func if hasattr(self, "hash_func") else lambda x: x
 
         def add_ctype_feats(arr, label, ctype):
             """a helper function to add all the feature values of ctype to arr
@@ -148,27 +158,26 @@ class RakutenMA(object):
                 arr.append(_f([label, ctype]))
             else:
                 arr += [_f([label, i]) for i in ctype]
-            return arr
 
-        for i in range(len(csent)):
+        for (i, x) in enumerate(csent):
             csent[i].f = []
 
             for feat in self.featset:
                 # character type unigram
                 if feat == "c0":
-                    csent[i].f = add_ctype_feats(csent[i].f, feat, _t(i).t)
+                    add_ctype_feats(csent[i].f, feat, _t(i).t)
                 elif feat == "c1":
-                    csent[i].f = add_ctype_feats(csent[i].f, feat, _t(i+1).t)
+                    add_ctype_feats(csent[i].f, feat, _t(i+1).t)
                 elif feat == "c9":
-                    csent[i].f = add_ctype_feats(csent[i].f, feat, _t(i-1).t)
+                    add_ctype_feats(csent[i].f, feat, _t(i-1).t)
                 elif feat == "c2":
-                    csent[i].f = add_ctype_feats(csent[i].f, feat, _t(i+2).t)
+                    add_ctype_feats(csent[i].f, feat, _t(i+2).t)
                 elif feat == "c8":
-                    csent[i].f = add_ctype_feats(csent[i].f, feat, _t(i-2).t)
+                    add_ctype_feats(csent[i].f, feat, _t(i-2).t)
                 elif feat == "c3":
-                    csent[i].f = add_ctype_feats(csent[i].f, feat, _t(i+3).t)
+                    add_ctype_feats(csent[i].f, feat, _t(i+3).t)
                 elif feat == "c7":
-                    csent[i].f = add_ctype_feats(csent[i].f, feat, _t(i-3).t)
+                    add_ctype_feats(csent[i].f, feat, _t(i-3).t)
                 # character unigram
                 elif feat == "w0":
                     csent[i].f.append(_f([feat, _t(i).c]))
@@ -220,7 +229,7 @@ class RakutenMA(object):
                 else:
                     # if the feature template is a function,
                     # invoke it and add the returned value
-                    if hasattr(feat, '__call__'):
+                    if hasattr(feat, "__call__"):
                         csent[i].f.append(_f(feat(_t, i)))
                     else:
                         raise ValueError("Invalid feature specification!")
@@ -241,17 +250,17 @@ class RakutenMA(object):
 
         for (i, cfeat) in enumerate(cfeats):
             cemits = Trie.find_partial(weights, cfeat) or e_def
-
-            # tag dictionary
-            # the possible set of tags is solely defined by the first feature
-            if i == 0:
-                states0.update({k: True for k in cemits})
-
-            if cemits:
-                for k in cemits:
-                    scores0[k] = scores0.get(k, 0) + cemits[k]['v']
+            for k in cemits:
+                if i == 0:
+                    # tag dictionary
+                    # the possible set of tags is solely defined by the first feature
+                    states0[k] = True
+                    scores0[k] = cemits[k]["v"]
+                else:
+                    scores0[k] = scores0.get(k, 0) + cemits[k]["v"]
         # replace by scores
-        states0.update(scores0)
+        for s0 in states0:
+            states0[s0] = scores0[s0]
         return states0
 
     def decode(self, csent):
@@ -262,15 +271,10 @@ class RakutenMA(object):
         Return:
             <list> csent
         """
-        t_def = Trie.insert({}, [_DEF_LABEL, _DEF_LABEL], 1.)
-        t_def = Trie.insert(t_def, [_BEOS_LABEL, _DEF_LABEL], 0.1)
-        t_def = Trie.insert(t_def, [_DEF_LABEL, _BEOS_LABEL], 0.1)
+        e_def = EDEF.copy()
 
-        e_def = Trie.insert({}, [_DEF_LABEL], 0.1)
-        e_def = Trie.insert(e_def, [_BEOS_LABEL], 0.)
-
-        weights = self.model.get('mu', {})
-        trans = weights.get('t', t_def)
+        weights = self.model.get("mu", {})
+        trans = weights.get("t", TDEF.copy())
 
         statesp = {
             _BEOS_LABEL: {"score": 0., "path": [_BEOS_LABEL]}
@@ -280,36 +284,31 @@ class RakutenMA(object):
 
         for char in csent[1:]:
             states0 = self.calc_states0(char.f, weights, e_def)
-            for s0 in states0:
+            for (s0, states0_score) in states0.items():
                 max_score = -float("inf")
                 max_state = None
-                states0_score = states0[s0]
                 trans0 = trans.get(s0, {})
                 for sp in statesp:
                     t_score = 0.
                     if sp in trans0:
-                        t_score = trans0[sp]['v'] or 0.
-
+                        t_score = trans0[sp]["v"]
+                    statesp_score = 0
                     if isinstance(statesp[sp], dict):
-                        statesp_score = statesp[sp]['score']
-                    else:
-                        statesp_score = 0
+                        statesp_score = statesp[sp]["score"]
                     score = statesp_score + states0_score + t_score
                     if score > max_score:
                         max_score = score
                         max_state = sp
-
                 if max_state and max_score > 0:
                     path = []
                     if isinstance(statesp[max_state], dict):
                         path = statesp[max_state]["path"]
-                    states0[s0] = {"score": max_score,
-                                   "path": path + [s0]}
+                    states0[s0] = {"score": max_score, "path": path + [s0]}
             statesp = states0
 
         # track the path and assign to csent[i].l
-        final_path = statesp[_BEOS_LABEL].get('path', {})
-        for i in range(len(csent)):
+        final_path = statesp[_BEOS_LABEL].get("path", {})
+        for (i, x) in enumerate(csent):
             csent[i].l = final_path[i] or _DEF_LABEL
         return csent
 
@@ -353,7 +352,7 @@ class RakutenMA(object):
                     tokens.append([cs.c, tail])
                     ctoken = None
         elif scheme == "IOB2":
-            for i in range(1, len(csent)-1):  # Skip BOS and EOS
+            for i in csent[1:-1]:  # Skip BOS and EOS
                 head = csent[i].l[0]
                 tail = csent[i].l[2:]
                 if head == "B":
@@ -400,15 +399,16 @@ class RakutenMA(object):
 
         if scheme == "SBIEO":
             for token in tokens:
-                if len(token[0]) == 1:
+                length = len(token[0])
+                if length == 1:
                     csent.append(Token(c=token[0], t=self.ctype_func(token[0]),
                                        l="S-" + token[1]))
                 else:
-                    for j in range(len(token[0])):
+                    for j in range(length):
                         tag = "I-"
                         if j == 0:
                             tag = "B-"
-                        elif j == len(token[0])-1:
+                        elif j == (length - 1):
                             tag = "E-"
                         csent.append(Token(c=token[0][j],
                                            t=self.ctype_func(token[0][j]),
@@ -450,37 +450,34 @@ class RakutenMA(object):
             <dict> res: result
         """
         res = {}
-        sent_str = "".join([i[0] for i in sent])
 
         # get answer feats
         ans_csent = self.tokens2csent(sent, self.tag_scheme)
         ans_csent = self.add_efeats(ans_csent)
         ans_feats = self.csent2feats(ans_csent)
-        ans_trie = {}
-        for ans_feat in ans_feats:
-            ans_trie = Trie.insert(ans_trie, ans_feat, 1)
-        res['ans'] = self.csent2tokens(ans_csent, self.tag_scheme)
+        res["ans"] = self.csent2tokens(ans_csent, self.tag_scheme)
 
         # get system output
-        sys_csent = self.str2csent(sent_str)
-        sys_csent = self.add_efeats(sys_csent)
-        sys_csent = self.decode(sys_csent)
-        sys_feats = self.csent2feats(sys_csent)
-        sys_trie = {}
-        for sys_feat in sys_feats:
-            sys_trie = Trie.insert(sys_trie, sys_feat, 1)
-        res['sys'] = self.csent2tokens(sys_csent, self.tag_scheme)
+        sys_csent = self.decode(ans_csent)
+        res["sys"] = self.csent2tokens(sys_csent, self.tag_scheme)
 
         # update
-        if self.tokens_identical(res['ans'], res['sys']):
-            res['updated'] = False
+        if self.tokens_identical(res["ans"], res["sys"]):
+            res["updated"] = False
         else:
+            ans_trie = {}
+            for ans_feat in ans_feats:
+                ans_trie = Trie.insert(ans_trie, ans_feat, 1)
             self.scw.update(ans_trie, 1)
-            self.scw.update(sys_trie, -1)
-            res['updated'] = True
 
-        self.model['mu'] = self.scw.mu
-        self.model['sigma'] = self.scw.sigma
+            sys_trie = {}
+            for sys_feat in self.csent2feats(sys_csent):
+                sys_trie = Trie.insert(sys_trie, sys_feat, 1)
+            self.scw.update(sys_trie, -1)
+
+            res["updated"] = True
+            self.model["mu"] = self.scw.mu
+            self.model["sigma"] = self.scw.sigma
         return res
 
     def prune(self, _lambda, sigma_th):
@@ -490,10 +487,9 @@ class RakutenMA(object):
             <float> _lambda
             <float> sigma_th
         """
-
         self.scw.prune(_lambda, sigma_th)
-        self.model['mu'] = self.scw.mu
-        self.model['sigma'] = self.scw.sigma
+        self.model["mu"] = self.scw.mu
+        self.model["sigma"] = self.scw.sigma
 
     def set_model(self, model):
         """set a new model
@@ -501,8 +497,8 @@ class RakutenMA(object):
             <dict> model
         """
         self.model = model
-        self.scw.mu = model.get('mu', {})
-        self.scw.sigma = model.get('sigma', {})
+        self.scw.mu = model.get("mu", {})
+        self.scw.sigma = model.get("sigma", {})
 
     @staticmethod
     def tokens2string(tokens):
@@ -580,7 +576,7 @@ class RakutenMA(object):
         tokenized_corpus = []
         for sent in corpus:
             sent_str = [j[0] for j in sent]
-            tokenized_corpus.append(tokenize_func(''.join(sent_str)))
+            tokenized_corpus.append(tokenize_func("".join(sent_str)))
         return tokenized_corpus
 
     @staticmethod
